import time

# Import the preset TestFixture.
from .Frames_capture_fixture import FixtureFramesCapture, ChannelFactory

from xtr import logging, Severity
logger = logging.getLogger(__name__)


# Create the test case class, that inherits from the preset TestFixture.
class TestCaptureFlexrayFrames(FixtureFramesCapture):

    # This method will be executed just before
    # the test case method execution.
    def setUp(self):
        logger.info("Setup test case")

    # This method will be executed just after
    # the test case method execution.
    def tearDown(self):
        logger.info("Tear down test case")

    # Create the test case method that holds the main test steps.
    def test_capture_flexray_frames(self):

        # Define frame properties.
        # The values provided below are arbitrary examples.
        # They do not belong to any actual inv ehicle communication.
        channel_name = 'B1_FlexRay'
        frame_id = '5.2.64'  # Its type is str.
        payload = [0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8,
                   0x9, 0xa, 0xb, 0xc, 0xd, 0xf, 0x10]

        flx_ch = ChannelFactory().get_flexray_channel(channel_name)

        flx_fr = flx_ch.get_frame(frame_id)

        # Start listening on the flexray channel.
        flx_ch.listener.start_listening(True)

        # Log any useful message.
        logger.info("Transmit a flexray frame")

        # Transmit the flexray frame.
        flx_fr.transmitter.transmit_frame(payload)

        # Pause the program for 3 seconds.
        time.sleep(3)

        # Stop listening on the flexray channel
        flx_ch.listener.stop_listening()

        q = flx_ch.listener.get_queue()

        # Confirm that the queue is not Null object,
        # meaning there is no error in creating the queue.
        self.assertTrue(q is not None, Severity.BLOCKER,
                        "Error when listening to flexray bus")

        # Confirm that queue is not empty meaning messages are received.
        self.assertTrue(not q.empty(), Severity.BLOCKER,
                        "Flexray frame is not received")

        event = q.queue[0]
        logger.info(repr(event))

        logger.info(f"{event.channel_name}")
        logger.info(f"{event.frame_id}")
